package;	

import hage.HGE;
import hage.HGESprite;
import hage.HGEVector2;

/**
 * ...
 * @author JanGames
 */
class Ball extends HGESprite
{
	/**Ball speed up*/
	private var speedup:Float;
	
	/**Initial ball position*/
	private var initPosition:HGEVector2;
	
	/**Initial ball veloicty*/
	private var initVelocity:HGEVector2;
	/**Ball velocity*/
	private var velocity:HGEVector2;
	
	/**Last paddle id for collision*/
	private var lastPaddleID:Int;
	
	public function new(X:Float = 0, Y:Float = 0)
	{
		super(X, Y, HGE.width / 20, HGE.height / 20, 0xFFFFFF00, 1);
		
		//Get initial position and velocity
		initPosition = new HGEVector2(X, Y);
		initVelocity = new HGEVector2(HGE.width / 5, HGE.height / 5);
		speedup = HGE.width / (HGE.width - HGE.width * 0.1);
		
		//Ball initial velocity
		velocity = new HGEVector2();
		
		//Reset to init values
		reset();
		
		//Get data, get paddle reference and add ball
		if(Data.instance != null) {
			Data.instance.balls.push(this);
		}
	}
	
	override public function update():Void {
		super.update();
		
		//Move with velocity
		x += velocity.x * HGE.delta;
		y += velocity.y * HGE.delta;
		
		//Bounce on walls
		if (x < 0 || x + width > HGE.width) {
			velocity.x *= -1;
			if (x < 0) { x = 0; }
			else { x = HGE.width - width; }
			
			//Add score on hit and reset
			if(Data.instance != null) {
				if (x < HGE.width / 2) { Data.instance.enemyPoints++; }
				else { Data.instance.playerPoints++; }
			}
			
			reset();
		}
		if (y < 0 || y + height > HGE.height) {
			velocity.y *= -1;			
			if (y < 0) { y = 0; }
			else { y = HGE.height - height; }			
		}
		
		//Bounce on paddles
		for (paddle in Data.instance.paddles) {
			if (aabb(paddle) && paddle.ID != lastPaddleID) {
				//Bounce ball
				if (x < HGE.width / 2) { velocity.x = Math.abs(velocity.x); }
				else { velocity.x = Math.abs(velocity.x) * -1; }
				
				//Speedup ball on hit
				velocity.mul(speedup);
				
				//Add paddle delta
				velocity.y += paddle.hdelta * 5;
				
				//Set paddle ID
				lastPaddleID = paddle.ID;
			}
		}
	}
	
	//Reset ball
	override public function reset() {
		super.reset();
		
		//Reset paddle id
		lastPaddleID = -1;
		
		//Reset position 	
		x = initPosition.x;
		y = initPosition.y;
		
		//Reset velocity
		velocity.set(initVelocity.x, initVelocity.y);
		var angle:Float = Math.random() * Math.PI * 0.5 + Math.PI * (0.5 + (Math.random() < 0.5 ? 1 : 0));	
		velocity.rotate(angle);
	}
	
}